#include-once
#region Header
#comments-start
	Title:			Timer Lib UDF Library for AutoIt3
	Filename:		TimerLib.au3
	Description:	A collection of UDF's that assist in defining several timers with function callbacks.
	Author:			J. DERN (JD) 
	Version:		01.00
	Last Update:	27.11.06
	Requirements:	AutoIt v3.2 +, Developed/Tested on WindowsXP Pro Service Pack 2
	Notes:			This code is optimized even with only one timer, it will takes less CPU if timer have a common divider
#comments-end
#endregion Header
#region Global Variables and Constants

Global $ClockArray[2][3]=[[0,0,0]]
Global $ClockCounter=0

;------------------------------------------------------------------------------------------------------
;-                                      D E M O N S T R A T I O N
;------------------------------------------------------------------------------------------------------
;~ ClockInit(1000, "A", true)
;~ ClockAdd (2000, "B", true)
;~ ClockAdd (5000, "C", True)
;~ ClockStartAll()
;~ While 1
;~   Sleep(10)
;~ WEnd

;~ Func A($timerno)
;~   ToolTip("A", 0, 0, "Time")
;~ EndFunc

;~ Func B($timerno)
;~   ToolTip("B", 50, 50, "Time")
;~ EndFunc

;~ Func C($timerno)
;~   ToolTip("C", 100, 100, "Time")
;~ EndFunc
;~ Exit


;------------------------------------------------------------------------------------------------------
;-                                      T I M E R   L I B R A R Y  
;------------------------------------------------------------------------------------------------------
;===============================================================================
; Description:      Initialize clock and add first timer.
; Parameter(s):     $frequence	- Call frequence in microseconds.
;					$function	- Function to call each $frequence microseconds (timer number is passed in argument)
;					$start		- Optional - True/False. True = start immediatly the timer
; Requirement(s):   AdlibEnable() or AdlibDisable() not used outside TimerLib
; Return Value(s):  The number of added timer
; Author(s):        Jerome DERN  (jdern "at" free "dot" fr)
; Note(s):			Multiple calls to this function will resets previously defined timers
;===============================================================================
Func ClockInit($frequence, $function, $start=True)
	
	; Initialize tables	
	$ClockArray[0][0] = 1			; Number of timers
	$ClockArray[0][1] = $frequence	; Actual frequency
	$ClockArray[0][2] = 1			; Max value of counter
	
	; Initialize first timer
	$ClockArray[1][0] = 1			; Number of Tick
	$ClockArray[1][1] = $function	; Function to call
	$ClockArray[1][2] = True		; Activated
	
	If $start Then ClockStartAll()
	Return 1
EndFunc

;===============================================================================
; Description:      Add a new timer.
; Parameter(s):     $frequence	- Call frequence in microseconds.
;					$function	- Function to call each $frequence microseconds
;					$start		- Optional - True/False. True = start immediatly the timer
; Requirement(s):   ClockInit() first called, AdlibEnable() or AdlibDisable() not used outside TimerLib
; Return Value(s):  The numer of added timer
; Author(s):        Jerome DERN  (jdern "at" free "dot" fr)
; Note(s):
;===============================================================================
Func ClockAdd($frequence, $function, $start=True)
	; ClockInit already called? If not initialize the first timer
	If $ClockArray[0][0] <= 0 Then Return ClockInit($frequence, $function, $start)
	
	AdlibDisable()								; Disable timer
	ReDim $ClockArray[$ClockArray[0][0]+2][3]	; Redim the timer table
	
	$nb = $ClockArray[0][0]						; Get number of timers
	
	; Convert nb of timeslot to timeslot duration
	$time = $ClockArray[0][1]					; Get actual timeslot duration
	For $i=1 to $nb
		$ClockArray[$i][0] *= $time				; = Nbslot*slotduration (timeslot duration)
	Next
	
	; Add new timer
	$nb += 1
	$ClockArray[$nb][0] = $frequence			; Number of clock tick to activate it
	$ClockArray[$nb][1] = $function				; Function to be called
	$ClockArray[$nb][2] = True					; Activated
	
	; Get maximum timeslot and common maximum divisor
	$time = $ClockArray[1][0]					; Get first timeslot duration
	$max = $time
	For $i=1 to $nb
		If $ClockArray[$i][0] > $max Then $max = $ClockArray[$i][0]		; Max?
		$time = ClockPGCD($time, $ClockArray[$i][0])	; Find greater common divisor
	Next
	
	; Convert back timeslot durations to number of timeslot
	For $i=1 to $nb
		$ClockArray[$i][0] /= $time				; Nb of clock tick
	Next
	
	; Update general informations
	$ClockArray[0][0] = $nb				; New number of timers
	$ClockArray[0][1] = $time			; Actual frequency
	$ClockArray[0][2] = $max/$time		; Max value of counter
	
	If $start Then ClockStartAll()
	Return $nb
EndFunc

;===============================================================================
; Description:      Start a timer given by its number
; Parameter(s):     $number		- The timer number to start, or callback function name.
; Requirement(s):   ClockInit() first called, AdlibEnable() or AdlibDisable() not used outside TimerLib
; Return Value(s):  1 if OK
;					-1 if no timer found
;					-2 if already started
; Author(s):        Jerome DERN  (jdern "at" free "dot" fr)
; Note(s):			$number is returned by ClockInit() or ClockAdd() services. Stop timer with ClockStop()
;===============================================================================
Func ClockStart($number)
	If Not IsNumber($number) And IsString($number) Then
		$name = $number
		For $i=1 to $ClockArray[0][0]
			If $ClockArray[$i][1] = $number Then
				$number = $i
				ExitLoop
			EndIf
		Next
	EndIf
	If $ClockArray[0][0] < $number Then Return -1
	If $ClockArray[$number][2] = True Then Return -2
	$ClockArray[$number][2] = True
	Return 1
EndFunc

;===============================================================================
; Description:      Stop a timer given by its number
; Parameter(s):     $number		- The timer number to stop, or callback function name.
; Requirement(s):   ClockInit() first called, AdlibEnable() or AdlibDisable() not used outside TimerLib
; Return Value(s):  1= 0K
;					-1 If no timer found
;					-2 If timer already stopped
; Author(s):        Jerome DERN  (jdern "at" free "dot" fr)
; Note(s):			$number is returned by ClockInit() or ClockAdd() services. Start again with ClockStart()
;===============================================================================
Func ClockStop($number)
	If Not IsNumber($number) And IsString($number) Then
		$name = $number
		For $i=1 to $ClockArray[0][0]
			If $ClockArray[$i][1] = $number Then
				$number = $i
				ExitLoop
			EndIf
		Next
	EndIf
	If $ClockArray[0][0] < $number Then Return -1
	If $ClockArray[$number][2] = False Then Return -2
	$ClockArray[$number][2] = False
	Return 1
EndFunc

;===============================================================================
; Description:      Stop all defined timers
; Parameter(s):     $number		- The timer number to stop.
; Requirement(s):   ClockInit() first called, AdlibEnable() or AdlibDisable() not used outside TimerLib
; Return Value(s):  None
; Author(s):        Jerome DERN  (jdern "at" free "dot" fr)
; Note(s):			ClockInit() already called. Start all timers again with ClockStartAll()
;===============================================================================
Func ClockStopAll()
	AdlibDisable()
EndFunc

;===============================================================================
; Description:      Start all defined timers
; Parameter(s):     $number		- The timer number to stop.
; Requirement(s):   ClockInit() first called, AdlibEnable() or AdlibDisable() not used outside TimerLib
; Return Value(s):  1 if OK, -1 if ClockInit() not called or no timer found
; Author(s):        Jerome DERN  (jdern "at" free "dot" fr)
; Note(s):			ClockInit() already called. Stop all services by ClockStopAll()
;===============================================================================
Func ClockStartAll()
	If $ClockArray[0][0] < 1 Then Return -1
	If $ClockArray[0][0] = 1 Then
		AdlibEnable($ClockArray[1][1], $ClockArray[0][1])
	Else
		AdlibEnable("ClockCallback", $ClockArray[0][1])
	EndIf
	Return 1
EndFunc

;===============================================================================
; Description:      Debug: print in tooltips all timers and their status
; Parameter(s):     none
; Requirement(s):   ClockInit() first called, AdlibEnable() or AdlibDisable() not used outside TimerLib
; Return Value(s):  number of timers defined
; Author(s):        Jerome DERN  (jdern "at" free "dot" fr)
; Note(s):			For debugging purposed no use in production code
;===============================================================================
Func ClockDebug()
	$info = $ClockArray[0][0] & " Clock defined" & @CRLF
	For $i=1 to $ClockArray[0][0]
		$period = $ClockArray[$i][0] * $ClockArray[0][1]
		$h = Int($period / (60*60000))
		$period -= $h*60*60000
		$m = Int($period/60000)
		$period -= $m*60000
		$s = Int($period/1000)
		$ms = $period - $s*1000
		$period = ""
		if $h >0 then $period = $period & $h & "h "
		if $m >0 then $period = $period & $m & "m "
		if $s >0 then $period = $period & $s &"s "
		if $ms >0 then $period = $period & $ms & "ms "
		$info = $info & " Clock " & $i & " status is " & $ClockArray[$i][2] & " at " & $period & "Callback is " & $ClockArray[$i][1] & @CRLF
	Next
	ToolTip($info, 0, 0, "ClockLib debug")
	Return $ClockArray[0][0]
EndFunc

;------------------------------------------------------------------------------------------------------
;-                  U T I L I T A R Y   F U N C T I O N S   N O T   E X P O R T E D  
;------------------------------------------------------------------------------------------------------
; The timer adlib general callback
Func ClockCallback()
	$ClockCounter += 1
	For $i=1 to $ClockArray[0][0]
		If Not $ClockArray[$i][2] Then ContinueLoop 
		If $ClockArray[$i][0]=1 Or Mod($ClockCounter, $ClockArray[$i][0]) = 0 Then Call($ClockArray[$i][1], $i)
	Next
	If $ClockCounter >= $ClockArray[0][2] Then $ClockCounter = 0
EndFunc

; Compute higher common divisor of two numbers
Func ClockPGCD($a, $b)
	If $b=0 Then 
		$ClockPGCD=$a
	Else
		$c = $a/$b
		$c = Round(($c-int($c))*$b, 0)
		$ClockPGCD = ClockPGCD($b, $c)
	EndIf
	Return $ClockPGCD
EndFunc